﻿using Hotline.FlowEngine.Workflows;
using Hotline.Repository.SqlSugar.DataPermissions;
using SqlSugar;
using XF.Domain.Entities;

namespace Hotline.Repository.SqlSugar.Extensions
{
    public static class DataPermissionExtensions
    {
        public static ISugarQueryable<TEntity> DataPermissionFiltering<TEntity>(this ISugarQueryable<TEntity> queryable, IDataPermissionFilterBuilder dataPermissionFilterBuilder)
            where TEntity : class, IEntity<string>, IDataPermission, new()
        {
            return queryable.Where(dataPermissionFilterBuilder.Build<TEntity>());
        }

        public static ISugarQueryable<TEntity> WorkflowDataFiltering<TEntity>(this ISugarQueryable<TEntity> queryable, IDataPermissionFilterBuilder dataPermissionFilterBuilder)
            where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
        {
            //return queryable.LeftJoin(dataPermissionFilterBuilder.BuildIncludeFlowData1<TEntity>());
            return queryable.Where(dataPermissionFilterBuilder.BuildIncludeFlowData<TEntity>());
        }

        public static TEntity InitDatePermission<TEntity>(this TEntity entity, IDataPermissionManager dataPermissionManager)
            where TEntity : class, IEntity<string>, IDataPermission, new()
        {
            var (orgId, creatorId, areaId, creatorName, creatorOrgName) = dataPermissionManager.GetDataPermissionOptions();
            entity.CreateDataPermission(orgId, creatorId, areaId, creatorName, creatorOrgName);
            return entity;
        }

        public static TEntity AssignToCreator<TEntity>(this TEntity entity, IDataPermissionManager dataPermissionManager)
            where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
        {
            var (orgId, creatorId, areaId, creatorName, creatorOrgName) = dataPermissionManager.GetDataPermissionOptions();
            entity.Assign(EFlowAssignType.User, creatorId);
            return entity;
        }
    }
}
