﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Settings;
using Hotline.Share.Requests;

namespace Hotline.Share.Dtos.Order
{
    public record QueryOrderDto : PagedKeywordRequest
    {

        /// <summary>
        /// 诉求内容
        /// </summary>
        public string? Content { get; set; }

        /// <summary>
        /// 受理类型
        /// </summary>
        public List<string> AcceptTypes { get; set; } = new();

        /// <summary>
        /// 来源渠道
        /// </summary>
        public List<string> Channels { get; set; } = new();

        /// <summary>
        /// 转接号码（转接来源）
        /// </summary>
        public string? TransferPhone { get; set; }

        /// <summary>
        /// 热点分类
        /// </summary>
        public List<string> HotspotIds { get; set; } = new();

        /// <summary>
        /// 接办部门
        /// </summary>
        public List<string> OrgCodes { get; set; } = new();

        /// <summary>
        /// 受理坐席名字或工号
        /// </summary>
        public string? NameOrNo { get; set; }

        /// <summary>
        /// 受理时间(工单创建时间)
        /// </summary>
        public DateTime? CreationTimeStart { get; set; }
        public DateTime? CreationTimeEnd { get; set; }

        /// <summary>
        /// 紧急程度
        /// </summary>
        public List<EEmergencyLevel> EmergencyLevels { get; set; } = new();

        /// <summary>
        /// 来电或联系电话
        /// </summary>
        public string? PhoneNo { get; set; }

        /// <summary>
        /// 推送分类
        /// </summary>
        public string? PushTypeCode { get; set; }

        /// <summary>
        /// 过期时间
        /// </summary>
        public DateTime? ExpiredTimeStart { get; set; }
        public DateTime? ExpiredTimeEnd { get; set; }

        /// <summary>
        /// 工单状态
        /// </summary>
        public List<EOrderStatus> Statuses { get; set; } = new();
    }


    public record QueryOrderPublishDto : PagedKeywordRequest
    {
        /// <summary>
        /// 发布状态
        /// </summary>
        public EPubState? PubState { get; set; }

        /// <summary>
        /// 工单标题
        /// </summary>
        public string? OrderTitle { get; set; }

        /// <summary>
        /// 发布人
        /// </summary>
        public string? PubMan { get; set; }

        /// <summary>
        /// 发布范围
        /// </summary>
        public EPublicState? PubRange { get; set; }

        /// <summary>
        /// 受理类型
        /// </summary>
        public List<string>? AcceptTypes { get; set; } = new();

        /// <summary>
        /// 热点分类
        /// </summary>
        public List<string>? HotspotIds { get; set; } = new();

        /// <summary>
        /// 受理时间(工单创建时间)
        /// </summary>
        public DateTime? CreationTimeStart { get; set; }
        public DateTime? CreationTimeEnd { get; set; }

        /// <summary>
        /// 归档时间
        /// </summary>
        public DateTime? FiledTimeStart { get; set; }
        public DateTime? FiledTimeEnd { get; set; }

    }

    public record QueryOrderVisitDto : PagedKeywordRequest
    {
        public EVisitStateQuery VisitState { get; set; }
    }

    public record VisitDetailListDto : PagedKeywordRequest
    {
        public EVisitStateQuery VisitState { get; set; }
    }

    public record VisitDto
    {
        /// <summary>
        /// 回访主表主键
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// 是否接通
        /// </summary>
        public bool IsPutThrough { get; set; }

        /// <summary>
        /// 是否重办
        /// </summary>
        public bool IsAgain { get; set; }

        public List<VisitDetailDto> VisitDetails { get; set; }
    }

    public record VisitDetailDto
    {
        /// <summary>
        /// Id
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// 回访主表ID
        /// </summary>
        public string VisitId { get; set; }

        /// <summary>
        /// 语音评价（话务评价）
        /// </summary>
        public EVoiceEvaluate? VoiceEvaluate { get; set; }
                               
        /// <summary>
        /// 话务员评价（话务评价）
        /// </summary>
        public ESeatEvaluate? SeatEvaluate { get; set; }

        /// <summary>
        /// 部门办件结果
        /// </summary>
        public IdName? OrgProcessingResults { get; set; }

        /// <summary>
        /// 不满意原因
        /// </summary>
        public List<IdName>? OrgNoSatisfiedReason { get; set; }

        /// <summary>
        /// 部门办件态度
        /// </summary>
        public IdName? OrgHandledAttitude { get; set; }

        /// <summary>
        /// 回访内容
        /// </summary>
        public string? VisitContent { get; set; }

        /// <summary>
        /// 回访部门名称
        /// </summary>
        public string? VisitOrgName { get; set; }

        /// <summary>
        /// 回访部门Code
        /// </summary>
        public string? VisitOrgCode { get; set; }

        /// <summary>
        /// 回访对象类型 10：话务员 20：部门
        /// </summary>
        public EVisitTarget? VisitTarget { get; set; }
    }

    public record QueryOrderRedoRecordDto: PagedKeywordRequest
    {

    }


    public record CanDelayOrderListDto : PagedKeywordRequest
    {

    }


    public record ApplyDelayDto
    {
        public string OrderId { get; set; }

        public int DelayNum { get; set; }

        public ETimeType DelayUnit { get; set; }

        public string DelayReason { get; set; }

        public List<string> FileIds { get; set; }
    }

    public record DelayCalcEndTimeDto
    {
        public DateTime BeginTime { get; set; }

        public int DelayNum { get; set; }

        public ETimeType DelayUnit { get; set; }
    }

    public record DelayListDto : PagedKeywordRequest
    {
        /// <summary>
        /// 是否已办 true:已办 false:待办
        /// </summary>
        public bool IsApply { get; set; }

        public EDelayState? DelayState { get; set; }
    }

    public record ApplySuperviseDto
	{
	    public string OrderId { get; set; }
    }

    public record SuperviseListDto : PagedKeywordRequest
    {
	    /// <summary>
	    /// 督办回复状态
	    /// </summary>
	    public int? SuperviseState { get; set; }

	    public DateTime? CreationTimeStart { get; set; }
	    public DateTime? CreationTimeEnd { get; set; }
	}

    public record ScreenListDto : PagedKeywordRequest
    {
		/// <summary>
		/// 甄别申请状态
		/// </summary>
		public EScreenStatus Status { get; set; }

		public DateTime? CreationTimeStart { get; set; }
	    public DateTime? CreationTimeEnd { get; set; }
    }

    public record MayScreenListDto : PagedKeywordRequest
    {
	    /// <summary>
	    /// 部门编号
	    /// </summary>
	    public string VisitOrgCode { get; set; }

	    public DateTime? CreationTimeStart { get; set; }
	    public DateTime? CreationTimeEnd { get; set; }
    }

	public record UrgeListDto : PagedKeywordRequest
    {
	    /// <summary>
	    /// 督办回复状态
	    /// </summary>
	    public int? SuperviseState { get; set; }
	    public DateTime? CreationTimeStart { get; set; }
	    public DateTime? CreationTimeEnd { get; set; }
    }

	public enum EVisitStateQuery
    {
        /// <summary>
        /// 全部
        /// </summary>
        [Description("全部")]
        All = 0,

        /// <summary>
        /// 已回访
        /// </summary>
        [Description("已回访")]
        Visited = 1,

        /// <summary>
        /// 未回访
        /// </summary>
        [Description("未回访")]
        NoVisit = 2,
    }


    public enum EPublicState
    {
        /// <summary>
        /// 全部
        /// </summary>
        [Description("全部")]
        All = 0,
        /// <summary>
        /// 公开
        /// </summary>
        [Description("公开")]
        Pub = 1,
        /// <summary>
        /// 不公开
        /// </summary>
        [Description("不公开")]
        NoPub = 2,
    }


    public enum EPubState
    {
        /// <summary>
        /// 全部
        /// </summary>
        [Description("全部")]
        All = 0,
        /// <summary>
        /// 已发布
        /// </summary>
        [Description("已发布")]
        Pub = 1,
        /// <summary>
        /// 未发布
        /// </summary>
        [Description("未发布")]
        NoPub = 2
    }

}
