﻿using Hotline.Share.Dtos.Settings;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Settings;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Settings.TimeLimits
{
    public class TimeLimitDomainService : ITimeLimitDomainService, IScopeDependency
    {
        private readonly ITimeLimitRepository _timeLimitRepository;
        private readonly IDaySettingRepository _daySettingRepository;
        private List<CalcTimeSetting> timeSettingList;
        private readonly IRepository<SystemSetting> _systemSettingRepository;

        public TimeLimitDomainService(ITimeLimitRepository timeLimitRepository,IDaySettingRepository daySettingRepository,IRepository<SystemSetting> systemSettingRepository)
        {
            _timeLimitRepository = timeLimitRepository;
            _daySettingRepository = daySettingRepository;
            _systemSettingRepository = systemSettingRepository;

            //初始化常量TODO
            timeSettingList = new List<CalcTimeSetting>();
            CalcTimeSetting setting1 = new CalcTimeSetting() { BugCode = "10", BusName = "咨询", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting1);
            CalcTimeSetting setting2 = new CalcTimeSetting() { BugCode = "30", BusName = "举报", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting2);
            CalcTimeSetting setting3 = new CalcTimeSetting() { BugCode = "35", BusName = "投诉", TimeType = ETimeType.WorkDay, TimeValue = 5 };
            timeSettingList.Add(setting3);
            CalcTimeSetting setting4 = new CalcTimeSetting() { BugCode = "20", BusName = "求助", TimeType = ETimeType.WorkDay, TimeValue = 5 };
            timeSettingList.Add(setting4);
            CalcTimeSetting setting5 = new CalcTimeSetting() { BugCode = "15", BusName = "建议", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting5);
            CalcTimeSetting setting6 = new CalcTimeSetting() { BugCode = "1", BusName = "意见", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting6);
            CalcTimeSetting setting7 = new CalcTimeSetting() { BugCode = "2", BusName = "惠民帮助", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting7);
            CalcTimeSetting setting8 = new CalcTimeSetting() { BugCode = "3", BusName = "大气污染举报", TimeType = ETimeType.WorkDay, TimeValue = 5 };
            timeSettingList.Add(setting8);
            CalcTimeSetting setting9 = new CalcTimeSetting() { BugCode = "25", BusName = "表扬", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting9);
            CalcTimeSetting setting10 = new CalcTimeSetting() { BugCode = "40", BusName = "其他", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting10);
        }

        public async Task<string> AddAsync(TimeLimit model, CancellationToken cancellationToken)
        {
            return await _timeLimitRepository.AddAsync(model, cancellationToken);
        }

        public async Task UpdateAsync(TimeLimit model, CancellationToken cancellationToken)
        {
            var dbModel = await _timeLimitRepository.GetAsync(model.Id, cancellationToken);
            if (dbModel == null)
            {
                throw new UserFriendlyException("无效数据");
            }
            await _timeLimitRepository.UpdateAsync(model, cancellationToken);
        }

        /// <summary>
        /// 计算工作时间分钟数
        /// </summary>
        /// <param name="beginTime"></param>
        /// <param name="endTime"></param>
        /// <param name="isCenter"></param>
        /// <returns></returns>
        public int CalcWorkTime(DateTime beginTime, DateTime endTime,bool isCenter)
        {
            if (isCenter)
            {
                //如果是中心，采用中心计算方式
                //计算时间差
                TimeSpan minuteSpan = new TimeSpan(endTime.Ticks - beginTime.Ticks);
                //总时差分钟数
                int minutes = (int)minuteSpan.TotalMinutes;


                //计算开始时间是否计算在工作时间内
                if (!IsWorkDay(beginTime))
                {
                    var newBeginTime = beginTime.AddDays(1).Date;
                    TimeSpan beginTimeMinuteSpan = new TimeSpan(newBeginTime.Ticks - beginTime.Ticks);
                    int beginTimeMinute = (int)beginTimeMinuteSpan.TotalMinutes;
                    minutes = minutes - beginTimeMinute;
                }


                //计算结束时间是否计算在工作时间内
                if (!IsWorkDay(endTime))
                {
                    var newEndTime = endTime.Date;
                    TimeSpan endTimeMinuteSpan = new TimeSpan(endTime.Ticks - newEndTime.Ticks);
                    int endTimeMinute = (int)endTimeMinuteSpan.TotalMinutes;
                    minutes = minutes - endTimeMinute;
                }

                //计算整天时间是否在工作时间内
                int days = minuteSpan.Days;
                for (int i = 0; i < days; i++)
                {
                    var newDay = beginTime.AddDays(i + 1);
                    if (newDay.Date == endTime.Date)
                    {
                        continue;
                    }

                    if (!IsWorkDay(beginTime.AddDays(i + 1)))
                    {
                        minutes = minutes - 1440;
                    }
                }
                return minutes;
            }
            else
            {
                //如果不是中心（则是部门计算方式）
                //
                var workTime = _systemSettingRepository.Get(x => x.Code == "WorkTime");
                if (workTime!=null)
                {
                    DateTime WorkBeginTime = DateTime.Parse(DateTime.Now.ToShortDateString() + " " + workTime.SettingValue[0] + ":00");
                    DateTime WorkEndTime = DateTime.Parse(DateTime.Now.ToShortDateString() + " " + workTime.SettingValue[1] + ":00");

                    //处理开始时间
                    if (beginTime < WorkBeginTime)
                    {
                        beginTime = WorkBeginTime;
                    }
                    else
                    {
                        if (beginTime > WorkEndTime)
                        {
                            beginTime = WorkBeginTime.AddDays(1);
                        }
                    }

                    int minutes = 0;
                    int i = 0;
                    while(true)
                    {
                        i++;
                        //如果是工作日
                        if (IsWorkDay(beginTime))
                        {
                            //最后一天
                            if (beginTime.Date == endTime.Date)
                            {
                                if (endTime > WorkEndTime)
                                {
                                    if (i!=1)
                                    {
                                        TimeSpan sp = new TimeSpan(WorkEndTime.Ticks - WorkBeginTime.Ticks);
                                        minutes += (int)sp.TotalMinutes;
                                    }
                                    else
                                    {
                                        TimeSpan sp = new TimeSpan(WorkEndTime.Ticks - beginTime.Ticks);
                                        minutes += (int)sp.TotalMinutes;
                                    }
                                }
                                else
                                {
                                    if (i!=1)
                                    {
                                        TimeSpan sp = new TimeSpan(endTime.Ticks - WorkBeginTime.Ticks);
                                        minutes += (int)sp.TotalMinutes;
                                    }
                                    else
                                    {
                                        TimeSpan sp = new TimeSpan(endTime.Ticks - beginTime.Ticks);
                                        minutes += (int)sp.TotalMinutes;
                                    }
                                }
                                return minutes;
                            }
                            //如果不是当天完成
                            else
                            {
                                if (i==1)
                                {
                                    TimeSpan sp = new TimeSpan(WorkEndTime.Ticks - beginTime.Ticks);
                                    minutes += (int)sp.TotalMinutes;
                                }
                                else
                                {
                                    if (endTime.Date == WorkEndTime.Date)
                                    {
                                        if(endTime > WorkEndTime)
                                        {
                                            TimeSpan sp = new TimeSpan(WorkEndTime.Ticks - WorkBeginTime.Ticks);
                                            minutes += (int)sp.TotalMinutes;
                                        }
                                        else
                                        {
                                            TimeSpan sp = new TimeSpan(endTime.Ticks - WorkBeginTime.Ticks);
                                            minutes += (int)sp.TotalMinutes;
                                        }
                                        return minutes;
                                    }
                                    else
                                    {
                                        TimeSpan sp = new TimeSpan(WorkEndTime.Ticks - WorkBeginTime.Ticks);
                                        minutes += (int)sp.TotalMinutes;
                                    }
                                }
                            }
                        }
                        
                        beginTime = beginTime.AddDays(1);
                        WorkEndTime = WorkEndTime.AddDays(1);
                        WorkBeginTime = WorkBeginTime.AddDays(1);
                    }
                }
                return 0;
            }
        }


        /// <summary>
        /// 计算工作日
        /// </summary>
        /// <param name="beginTime"></param>
        /// <param name="busCode"></param>
        /// <param name="isCenter"></param>
        /// <returns></returns>
        public TimeResult? CalcEndTime(DateTime beginTime, string busCode,bool isCenter)
        {
            var setting = timeSettingList.FirstOrDefault(x => x.BugCode == busCode);
            if (setting != null)
            {
                return CalcEndTime(beginTime, setting.TimeType, setting.TimeValue,isCenter);
            }
            return null;
        }


        /// <summary>
        /// 计算工作日
        /// </summary>
        /// <param name="beginTime"></param>
        /// <param name="timeType"></param>
        /// <param name="timeValue"></param>
        /// <param name="isCenter"></param>
        /// <returns></returns>
        public TimeResult CalcEndTime(DateTime beginTime, ETimeType timeType, int timeValue,bool isCenter)
        {
           
            //如果是部门，采用部门计算方式
            switch (timeType)
            {
                //新增对应小时
                case ETimeType.Hour:
                    if (isCenter)
                    {
                        //中心计算自然时间
                        return new TimeResult { EndTime = beginTime.AddHours(timeValue), RuleStr = timeValue + "小时" };
                    }
                    else
                    {
                        //部门计算工作时间
                        //查询统一部门工作时间
                        var workTime = _systemSettingRepository.Get(x => x.Code == "WorkTime");
                        if (workTime!=null)
                        {
                            //计算一天工作时间（分钟）
                            DateTime WorkBeginTime =  DateTime.Parse(DateTime.Now.ToShortDateString()+" "+workTime.SettingValue[0]+":00");
                            DateTime WorkEndTime = DateTime.Parse(DateTime.Now.ToShortDateString() + " " + workTime.SettingValue[1] + ":00");
                            TimeSpan minuteSpan = new TimeSpan(WorkEndTime.Ticks - WorkBeginTime.Ticks);
                            //一天工作时间（分钟）
                            int minutes = (int)minuteSpan.TotalMinutes;
                            //延期总时间（分钟）
                            int timeValueMinutes = timeValue * 60;
                            
                            //判断开始时间不在工时间段修正时间复位至当天或者第二天的开始时间（如果大于结束时间则复位至第二天的开始时间）
                            if (beginTime < WorkBeginTime)
                            {
                                beginTime = WorkBeginTime;
                            }
                            else
                            {
                                if (beginTime>WorkEndTime)
                                {
                                    beginTime = WorkBeginTime.AddDays(1);
                                }
                            }

                            //判断第一天时间是否足够
                            TimeSpan firstMinuteSpan = new TimeSpan(WorkEndTime.Ticks - beginTime.Ticks);
                            //计算开始时间的第一天时间
                            int firstMinute = (int)firstMinuteSpan.TotalMinutes;
                            //判断第一天时间是否足够
                            if (firstMinute> timeValueMinutes)
                            {
                                //足够
                                bool canWhile = true;
                                while (canWhile)
                                {
                                    //判断开始时间是否在工作日
                                    if (IsWorkDay(beginTime))
                                    {
                                        return new TimeResult { EndTime = beginTime.AddHours(timeValue), RuleStr = timeValue + "小时" };
                                    }
                                    else
                                    {
                                        //如果不是工作日就加一天
                                        beginTime = beginTime.AddDays(1);
                                    }
                                }
                                return new TimeResult { EndTime = beginTime.AddHours(timeValue), RuleStr = timeValue + "小时" };
                            }
                            else
                            {
                                //不够
                                bool canWhile = true;
                                while (canWhile)
                                {
                                    //判断开始时间是否在工作日
                                    if (IsWorkDay(beginTime))
                                    {
                                        //在工作日
                                        timeValueMinutes = timeValueMinutes - minutes;
                                        if (timeValueMinutes>0)
                                        {
                                            beginTime = beginTime.AddDays(1);
                                        }
                                        else
                                        {
                                            return new TimeResult { EndTime = beginTime.AddMinutes(timeValueMinutes), RuleStr = timeValue + "小时" };
                                        }
                                    }
                                    else
                                    {
                                        //如果不是工作日
                                        beginTime = beginTime.AddDays(1);
                                    }
                                }
                            }

                        }
                        return new TimeResult { EndTime = beginTime.AddHours(timeValue), RuleStr = timeValue + "小时" };
                    }
                        
                //新增工作日
                case ETimeType.WorkDay:
                    //检查时间段内是否存在休息日或者工作日
                    //查询统一部门工作时间
                    var workTimeWorkDay = _systemSettingRepository.Get(x => x.Code == "WorkTime");
                    if (workTimeWorkDay != null)
                    {
                        //计算一天工作时间（分钟）
                        DateTime WorkBeginTime = DateTime.Parse(DateTime.Now.ToShortDateString() + " " + workTimeWorkDay.SettingValue[0] + ":00");
                        DateTime WorkEndTime = DateTime.Parse(DateTime.Now.ToShortDateString() + " " + workTimeWorkDay.SettingValue[1] + ":00");

                        //判断开始时间不在工时间段修正时间复位至当天或者第二天的开始时间（如果大于结束时间则复位至第二天的开始时间）
                        if (beginTime < WorkBeginTime)
                        {
                            beginTime = WorkBeginTime;
                        }
                        else
                        {
                            if (beginTime > WorkEndTime)
                            {
                                beginTime = WorkBeginTime.AddDays(1);
                            }
                        }

                        int day = 1;
                        for (int i = 1; i < timeValue + 1; i++)
                        {
                            if (IsWorkDay(beginTime.AddDays(day)))
                            {
                                beginTime = beginTime.AddDays(day);
                                day = 1;
                            }
                            else
                            {
                                i--;
                                day++;
                            }
                        }
                    }
                    return new TimeResult { EndTime = beginTime, RuleStr = timeValue + "个工作日" };
                //新增自然日
                case ETimeType.Day:
                    return new TimeResult { EndTime = beginTime.AddDays(timeValue), RuleStr = timeValue + "个自然日" };
                default:
                    return null;
            }
        }


        /// <summary>
        /// 计算时间间隔
        /// </summary>
        /// <param name="beginTime"></param>
        /// <param name="endTime"></param>
        /// <param name="isCenter"></param>
        /// <returns></returns>
        public string CalcTimeInterval(DateTime beginTime,DateTime endTime,bool isCenter)
        {
            if (isCenter)
            {
                //如果是中心按24小时制
                TimeSpan centerIntervalSp = new TimeSpan(endTime.Ticks - beginTime.Ticks);
                double re = Math.Round(((double)centerIntervalSp.Microseconds / (24 * 60)),1);
                if (re>0)
                {
                    return re+"天后超期";
                }
                else
                {
                    return "已超期" + Math.Abs(re) + "天";
                }
            }
            else
            {
                //如果是部门按工作时间

                return "";
            }
        }


        private bool IsWorkDay(DateTime date)
        {
            return _daySettingRepository.IsWorkDay(date).GetAwaiter().GetResult();
        }


    }
}
